#######################################################################
#
# ABOUT THIS FILE:  This file is intended to generate an animated flag
#                   above the Tux head.
#
# STEPS:  Several things must be done before this script will really do some work.
#
# 0. Blender must be started from FreeDroid/graphics/blender_sources directory.
#
# 1. Coppy (part of FreedroidRPG) must be compiled in FreeDroid/croppy/ dir.
#    (The path where to find croppy must be entered below in variable 'croppy_path')
#
# 2. /tmp/ directory must be writable for common user.  This is because
#    blender output will go directly to that dir before it is properly
#    cropped by the croppy tool and then placed in the proper subdir of
#    the FreeDroid directory tree.
# 
# 3. The visible layers for the rendering should be only the first 
#    layer.
#    (didn't find a means to set the visible layers from inside the script
#     and question in elysiun python forum didn't turn up anything either,
#     ...)
#
#--------------------
#
# COMMENTS:
#
# an occasional error message 'cannot stat ...0000.png' is completely ok.  
# The script should still work as intended.
#
#----------------------------------------------------------------------


######################################################################
######################### PREPARE EVERYTHING #########################
######################################################################

######################
# Let's start with the preparations:  Importing modules...
#
import Blender
import os
import sys
from math import *

######################
# Define some constants for later computation
#
pi2 = pi * 2 
number_of_tux_directions = 16.0
rot_angle = pi2 / number_of_tux_directions
our_cam = Blender.Object.Get("RotoCamera")

######################
# Define some file pathes...
#
croppy_path = "./../../croppy/croppy"
mv_path = "mv"
rm_path = "rm"
source_directory = "/tmp/"
target_directory = "./../"

######################################################################
################## FIND OUT THE OUTPUT DIRECTION #####################
######################################################################

######################
# Now we can start to identify the current direction
# by taking a closer look at the current camera position.
#
output_direction = 0 
start_x_pos = our_cam.LocX
start_y_pos = our_cam.LocY
#
# Now we need to be careful to avoid division by zero...
#
if our_cam.LocX == 0:
	our_arctan = atan ( our_cam.LocY / 0.0001 )
else:
	our_arctan = atan ( our_cam.LocY / our_cam.LocX )

def get_output_direction_16():
	global output_direction
	if abs( our_arctan - 0 * pi/4 ) < pi2/32:
		if start_x_pos < 0 :
			output_direction = 4
		else:
			output_direction = 12
	elif abs( our_arctan - 1 * pi/8 ) < pi2/32:
		if start_x_pos < 0 :
			output_direction = 3
		else:
			output_direction = 11
	elif abs( our_arctan + 1 * pi/8 ) < pi2/32:
		if start_x_pos < 0 :
			output_direction = 5
		else:
			output_direction = 13
	elif abs( our_arctan - 1 * pi/4 ) < pi2/32:
		if start_x_pos < 0 :
			output_direction = 2
		else:
			output_direction = 10
	elif abs( our_arctan + 1 * pi/4 ) < pi2/32:
		if start_y_pos < 0 :
			output_direction = 14
		else:
			output_direction = 6
	elif abs( our_arctan + 3 * pi/8 ) < pi2/32:
		if start_y_pos < 0 :
			output_direction = 15
		else:
			output_direction = 7
	elif abs( our_arctan - 3 * pi/8 ) < pi2/32:
		if start_y_pos < 0 :
			output_direction = 1
		else:
			output_direction = 9
	elif abs( our_arctan - 2 * pi/4 ) < pi2/32:
		if start_y_pos < 0 :
			output_direction = 0
		else:
			output_direction = 8
	elif abs( our_arctan + 2 * pi/4 ) < pi2/32:
		if start_y_pos < 0 :
			output_direction = 0
		else:
			output_direction = 8
	else:
		print "ERROR:  UNABLE TO DETERMINE DIRECTION!"
		Exit()

get_output_direction_16()
print "BLENDER (MAIN): Final output direction: " , output_direction

######################################################################
##################### CAMERA ROTATION FUNCTION #######################
######################################################################

#######################
# Now we define our function that will rotate the camera
# by a fraction of a full turn
#
def do_cam_rot():
	r = our_cam.LocX * our_cam.LocX + our_cam.LocY * our_cam.LocY
	# print "Radius square: " , r
	r = sqrt ( r )
	# print "Radius used (without Z component):" , r
	######################
	# First we print out the starting position and rotation of the camera, just
	# so that we can check everything is all right, if we want to...
	#
	# print "Starting with coordinates: " , our_cam.loc
	# print "Starting with rotation: " , our_cam.rot
	######################
	# Now it's time to start doing the rotation.  Hope everything will go well...
	#
	X_new = 1.0 * ( cos ( rot_angle ) * our_cam.LocX + sin ( rot_angle ) * our_cam.LocY )
	Y_new = 1.0 * (-sin ( rot_angle ) * our_cam.LocX + cos ( rot_angle ) * our_cam.LocY )
	
	our_cam.LocX = X_new
	our_cam.LocY = Y_new
	our_cam.RotZ = our_cam.RotZ - rot_angle
	Blender.Redraw()

######################################################################
###################### RENAME AND CROPPY FUNCTION ####################
######################################################################

########################
# Now we check if maybe the last frame of the animation
# has been reached.  In this case we will do a rotation 
# of the camera.  If not we will start a croppy operation
# on the current output file.
#
time = Blender.Get("curframe") - 1

def do_rename_and_croppy():
	global output_direction
	global subdirectory_string
	global body_part_string
	global target_directory
	#
	#--------------------
	# Now we find out the number code of the source image file
	# on the disk, that has been created by the blender by now.
	# With this we can fix the source file name now.
	#
	if time < 10 :
		number_code = "0" + "0" + "0" + str(time)
	elif time < 100 :
		number_code = "0" + "0" + str(time)
	elif time < 1000 :
		number_code = "0" + str(time)
	else :
		number_code = str(time)
	source_filename = source_directory + "iso_gun_tmpfile_" + number_code + ".png"
	print "BLENDER (do_rename_and_croppy): Source file name: " , source_filename
	#
	#--------------------
	# Now we find out the end of the target file name, i.e. the last number code.
	# We introduce some redundancy to be safe against some last or first image in
	# the series getting lost somehow.  That makes up 3% of extra redundant computation
	# but that should be a tolerable price.
	#
	target_time = ( ( time ) ) + 1
	if target_time < 10 :
		target_number_code = "0" + "0" + "0" + str(target_time)
	else :
		target_number_code = "0" + "0" + str(target_time)
	#
	#--------------------
	# Now we fix the output number code.  The output number code can be derived from
	# the current camera position.
	#
	get_output_direction_16()
	if output_direction < 10 :
		direction_code = "0" + str ( output_direction )
	else:
		direction_code = str ( output_direction )
	#
	#--------------------
	# Now we should have all the information we need to assemble the final target
	# file name.
	#
	target_filename = target_directory
	target_filename = target_filename + "iso_" + body_part_string + "_" + direction_code
	target_filename = target_filename + "_" + target_number_code + ".png"
	print "BLENDER (do_rename_and_croppy): target file name would be: " , target_filename
	#
	#--------------------
	# Now we can start the croppy operation.
	#
	if ( ( target_time != 0 ) & ( target_time <= stop_file_generation_after_image_nr ) ) :
		# os.system ( "pwd" )
		command_line = mv_path + " " + source_filename + " " + target_filename
		print "BLENDER (do_rename_and_croppy): invoking 1st command line: " , command_line
		os.system ( command_line )
		# ----
		parameter = " -i " + target_filename
		print "BLENDER (do_rename_and_croppy): invoking 2nd command line: " , croppy_path , parameter
		os.system ( croppy_path + parameter )
	else :
		print "BLENDER (do_rename_and_croppy): This image will not be needed by the engine "
		print "                                and therefore we delete it."
		command_line = rm_path + " " + source_filename
		os.system ( command_line )
		# ----
	print " "

######################################################################
###################### ACTIVATION OF A BODY PART #####################
######################################################################

########################
# For a render-everything run we need to activate one body part.
# The part to be activated depends on the current time.
#

def set_current_part_string() :
	global body_part_string
	global time
	part_code = int ( ( int ( time + 0 ) ) / ( int ( ( full_motion_cycle ) * number_of_tux_directions ) ) )
	if ( part_code == (-1) ) :
		body_part_string = "head"
		# use images 0-576 for this
	elif ( part_code == 0 ) :
		body_part_string = "head"
		# use images 0-576 for this
	elif ( part_code == 1 ) :
		body_part_string = "torso"
		# use images 576-1152 for this
	elif ( part_code == 2 ) :
		body_part_string = "helm1"
		# use images 1152-1728 for this
	elif ( part_code == 3 ) :
		body_part_string = "armour1"
		# use images 1728-2304 for this
	elif ( part_code == 4 ) :
		body_part_string = "robe"
		# use images 2304-2880 for this
	elif ( part_code == 5 ) :
		body_part_string = "shieldarm"
		# use images 2880-3456 for this
	elif ( part_code == 6 ) :
		body_part_string = "gun1" 
		# use images 3456-4032 for this
	elif ( part_code == 7 ) :
		body_part_string = "feet"
		# use images 4032-4608 for this
	elif ( part_code == 8 ) :
		body_part_string = "boots1"
		# use images 4608-5184 for this
	elif ( part_code == 9 ) :
		body_part_string = "weaponarm"
		# use images 5184-5760 for this
	else :
		print "********************"
		print "*** END OF ULTIMATE TUX GUN MOTION RENDERING REACHED"
		print "********************"
		print " "
		print " Should be safe to terminate now... Terminating..."
		print "...trying 'sys.exit()' now..."
		# os.abort()
		# sys.exit()
		print "...obviously, that hasn't worked. "
		print " ... Termination seems not to work... using safe part name 'NOTHING'"
		print " "
		body_part_string = "NOTHING"
	print "BLENDER (set_current_part_string) : The part code calculated was: " , part_code

######################################################################
########################## 'MAIN PROGRAM' ############################
######################################################################

#--------------------
# Now we check if maybe all the animation in the current direction has
# been properly recorded and if therefore it's time to rotate the camera 
# to a new position for a new run of the animation in a new direction.
#
if ( time == 0 ) :
	do_cam_rot()
	do_rename_and_croppy()
else:
	prepare_scene_for_tux_part()
	do_rename_and_croppy()